--Obtén las actividades de tus aplicaciones
require "import"
import "android.content.Intent"
import "android.content.pm.ApplicationInfo"
import "android.content.Context"
import "android.content.pm.PackageManager"
import "android.widget.ArrayAdapter"
import "android.widget.GridView"
import "android.widget.LinearLayout"
import "android.widget.TextView"
import "android.widget.Toast"
import "android.widget.EditText"
import "android.os.Vibrator"
import "android.view.Gravity"
import "com.androlua.*"

local context = activity or service
local packageManager = context.getPackageManager()
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)

local selectedApp = nil

--Marcador  Función para abrir la aplicación seleccionada de forma normal
-- Función para abrir la aplicación seleccionada y mostrar actividades públicas
local function openSelectedApp()
    if selectedApp then
        local packageName = selectedApp.packageName
        local mainIntent = context.getPackageManager().getLaunchIntentForPackage(packageName)

        if mainIntent then
            local packageInfo = packageManager.getPackageInfo(packageName, PackageManager.GET_ACTIVITIES)

            if packageInfo.activities then
                local publicActivities = {}
                local activitiesLength = #packageInfo.activities

                -- Imprimir el nombre del paquete en el encabezado
print("plantilla")
print([[require "import"
import "android.content.*"
intent = Intent(Intent.ACTION_MAIN); intent.setComponent(ComponentName(
"]] .. packageName .. [[",
""
)) this.startActivity(intent)
return true
]])
print("actividades públicas")

                for i = 0, activitiesLength - 1 do
                    local activityInfo = packageInfo.activities[i]
                    if activityInfo.exported then
                        publicActivities[#publicActivities + 1] = activityInfo.name
                    end
                end

                -- Imprimir la lista de actividades públicas
                for _, activity in ipairs(publicActivities) do
                    print(activity)
                end
            else
                print("No se encontraron actividades para la aplicación seleccionada")
            end
        else
            print("No se encontró el intent de lanzamiento para la aplicación seleccionada")
        end
    else
        Toast.makeText(context, "Debes seleccionar una aplicación", Toast.LENGTH_SHORT).show()
    end
end
-- Obtener la lista de aplicaciones instaladas
local installedApplications = packageManager.getInstalledApplications(PackageManager.GET_META_DATA)

-- Crear una lista de nombres de aplicaciones y un diccionario de aplicaciones con nombres correspondientes
local appNames = {}
local installedAppInfos = {}
for i = 1, installedApplications.size() do
    local appInfo = installedApplications.get(i - 1)
    if (appInfo.flags & ApplicationInfo.FLAG_SYSTEM) == 0 then
        local appName = appInfo.loadLabel(packageManager)
        local packageName = appInfo.packageName
        local intent = packageManager.getLaunchIntentForPackage(packageName)
        if intent then
            installedAppInfos[appName] = appInfo
            appNames[#appNames + 1] = appName
        end
    end
end

-- Ordenar la lista de nombres de aplicaciones en orden alfabético
table.sort(appNames)

-- Función para filtrar la lista de aplicaciones
local function filterApps(query)
    local filteredApps = {}
    for _, appName in ipairs(appNames) do
        if string.match(string.lower(appName), string.lower(query)) then
            table.insert(filteredApps, appName)
        end
    end
    return filteredApps
end

-- Crear una vista de cuadrícula utilizando AndroLua
local gridView = GridView(context)
gridView.setNumColumns(1)
gridView.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT))
local adapter = ArrayAdapter(context, android.R.layout.simple_list_item_1, appNames)
gridView.setAdapter(adapter)

-- Crear un EditText para la búsqueda
local searchEditText = EditText(context)
searchEditText.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT))
searchEditText.setHint("Buscar...")

-- Crear una variable para mantener una referencia a la lista actual de aplicaciones
local currentApps = appNames

-- Agregar evento para detectar cambios en el EditText
searchEditText.addTextChangedListener({
    onTextChanged = function(s, start, before, count)
        local searchText = tostring(s)
        if #searchText > 0 then
            local filteredApps = filterApps(searchText)
            currentApps = filteredApps
            gridView.setAdapter(ArrayAdapter(context, android.R.layout.simple_list_item_1, filteredApps))
        else
            currentApps = appNames
            gridView.setAdapter(ArrayAdapter(context, android.R.layout.simple_list_item_1, appNames))
        end
    end
})

-- Crear un layout lineal vertical y agregar el título, el EditText y la vista de cuadrícula
local linearLayout = LinearLayout(context)
linearLayout.setOrientation(LinearLayout.VERTICAL)
linearLayout.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT))

local titleTextView = TextView(context)
titleTextView.setText("Selecciona una aplicación")
titleTextView.setTextSize(20)
titleTextView.setGravity(Gravity.CENTER)
titleTextView.setPadding(16, 16, 16, 16)

linearLayout.addView(titleTextView)
linearLayout.addView(searchEditText)
linearLayout.addView(gridView)

-- Crear el diálogo utilizando LuaDialog y establecer el contenido como el layout lineal
local dlg = LuaDialog(context)
dlg.setTitle("Selecciona una aplicación")
dlg.setView(linearLayout)

-- Manejar el evento de clic en la cuadrícula
gridView.setOnItemClickListener{
    onItemClick = function(parent, view, position, id)
        local appName = currentApps[position + 1]
        vibrator.vibrate(100)
        Toast.makeText(context, " " .. appName, Toast.LENGTH_SHORT).show()
        selectedApp = installedAppInfos[appName]
        openSelectedApp()
        dlg.dismiss()
    end
}

dlg.show()